import numpy as np
 
# Import plotting functions
from matplotlib import pyplot as plt

# and matplotlib in general
import matplotlib
 
### Know your data format!  
# Check the buoy_data.csv file to see its header and formatting.

def read_data(fname):
    ''' Read the date column (0) and the data column from the file. '''
    # Numpy's genfromtxt can handle rows with missing data. It'll fill in 
    # 'np.nan' values.
    data = np.genfromtxt(fname,  # The filename to read
                     skip_header=1, # Ignore the header row
                     delimiter=',', # comma-separated file
                     encoding='latin1', # what sort of text is this?
                     converters={0: matplotlib.dates.datestr2num})
    # The converters dictionary says to convert column 0 using the 
    # matplotlib.dates.datestr2num function.
    #
    # datestr2num produces a date number: 735559 for 2015-01-01. Specifying
    # hours/minutes/seconds would make it a fractional number - 735559.59 etc.
    # see https://matplotlib.org/api/dates_api.html#matplotlib.dates.datestr2num
    # 
    # The default conversion (to a float) is applied to the rest of the
    # columns.
    # Why read it like this?  It makes it easy to do numeric calculations with
    # the date column if desired and all columns can be read into a single 
    # 2D float-type numpy matrix.
    return data


def condition_data(data):
    ''' Remove any rows that contain nan values. 
        data is a 2D matrix of date values in the
        first column and data values in the 2nd column'''
    # np.where applies the condition and returns indices where it's True.
    # np.isfinite is true when a number is not nan and not infinity
    idx = np.where(np.isfinite(data[:,1]))
    # idx is a 1-element tuple of the row numbers where there's a 
    # valid number in data's column 1.

    # Re-write the data variable using the "good" rows only.
    good_data = data[idx[0],:]
    
    # And now sort by date, the 1st column.
    # argsort returns the indices of the sorted order.
    # Use that to re-order good_data.
    idx = good_data[:,0].argsort()
    return good_data[idx]


if __name__=='__main__':
    # Set a filename.
    # These are from the NERACOOS ocean buoys, off the 
    # western coast of New England
    fname = 'buoy_data.csv'  
    # Get all the data.
    data = read_data(fname)
    
    # Get the 2m deep water temperatures 
    # for buoy A01 next to Gloucester,MA.  That's column 1
    buoy_a01 = data[:,(0,1)]
    
    # And how about the 2m deep temperatures
    # from buoy B01 in the Gulf of Maine. That's column 3
    buoy_b01 = data[:,(0,3)]
    
    # And now clean up both data sets
    buoy_a01 = condition_data(buoy_a01)
    buoy_b01 = condition_data(buoy_b01)

    # Let's plot!
    plt.plot(buoy_a01[:,0],buoy_a01[:,1],'b')
    plt.plot(buoy_b01[:,0],buoy_b01[:,1],'g')
    plt.xlabel('Day number')
    plt.ylabel('Deg. C')
    plt.legend(('A01','B01'))
    plt.show()
     
    